********************************************************************
*                                                                   
*       FORTRAN Package for Polarized  Structure Functions in
*                   Symmetry Breaking Scenario 		  
*           	  		  
*                                  
*                                                                   
*          F.Arbabifard ,A.Khorramian, S.Atashbar Tehrani
*
*       	 Ref: Submitted in Phys.Rev.D (2012).  
*
*          Comments to:                                          
*            							                                    
*               F. Arbabifar (Hamzeh.Farbabifar@ipm.ir)               
*               Ali N. Khorramian (Khorramiana@theory.ipm.ac.ir)      
*               S. Atashbar Tehrani (Atashbar@ipm.ir)                                         
*                                                                   
*                                                                   
*                                                                   
*                                                                   
*            INPUT:   iset = number of the parton set :             
*                                                                  
*                     iset = 1 NEXT LEADING ORDER                   
*                      data file: NLO_AKT.grid                       
*                                                                   
*            							                              	 
*                                                                   
*            x  = Bjorken-x       (between  1.0E-6 and 1.0E+0)      
*            q2 = scale in GeV**2 (between  0.1E+1 and 1.0E+6)      
*                                                                   
*   OUTPUT:  Uv   = x*DELTA u_v                                     
*            Dv   = x*DELTA d_v                                     
*            Gl   = x*DELTA gluon                                   
*            Sea  = x*DELTA qbar                                      
*            G1p  = x*g_1^proton                                    
*            G1n  = x*g_1^neutron                                    
*            G1d  = x*g_1^deuteron  
*	       G2   = x*g_2 
*            DUv   Error in Uv
*            DDv   Error in Dv
*            DUbar Error in Ubar
*            DDbar Error in Dbar
*            DSbar Error in Sbar
*            DGl   Error in Gl
*            dG1p  Error in G1p
*            dG1n  Error in G1n
*            dG1d  Error in G1d                        
*	       
*********************************************************************
*
      SUBROUTINE PPDF(ISET, X, Q2, Uv, Dv, Gl,
     1 Ubar, Dbar, Sbar , DUv, DDv, Dubar, Ddbar, Dsbar, DGl,
     1 G1p, G1n, G1d, G2, dG1p, dG1n, dG1d)
*     ---------------------------------------------------------------
*
      IMPLICIT REAL*8 (A-H,O-Z)
*     
      PARAMETER (NPDF=19, NQ=50, NX=100 )
*
      
*
      COMMON / INTINI / IINI
*
      DIMENSION QS(NQ),XB(NX),XPDF(NX,NQ,NPDF)
      DIMENSION AXB(NX,NQ,NPDF),BXB(NX,NQ,NPDF),CXB(NX,NQ,NPDF)
      
*
      DIMENSION PDF1(NPDF),PDF2(NPDF)
      DIMENSION PDF(NPDF)
*
      SAVE QS, XB, XPDF
      SAVE AXB, BXB, CXB

*     CHECK OF X AND Q2 VALUES : 
*
      IF ( X.LT.1.0D-06 .OR. X.GT.1.0D+00 ) THEN
         WRITE(6,2000) 
 2000    FORMAT (2X,'PARTON INTERPOLATION: X OUT OF RANGE -- STOP')
         STOP
      ENDIF
      IF ( Q2.LT.1.0D+00 .OR. Q2.GT.1.0D+06 ) THEN
         WRITE(6,2001) 
 2001    FORMAT (2X,'PARTON INTERPOLATION: Q2 OUT OF RANGE -- STOP')
         STOP
      ENDIF
*
      IF (IINI .NE. 0) GOTO 10
*
*     READING OF THE DATA SET :
*     
      LIN = 10
*
      IF (ISET .EQ. 1) THEN
*
*     open input unit for LO data set N (Scenario 1)
*
         OPEN(UNIT=LIN,FILE='./NLO_AKT.grid',
     1              STATUS='UNKNOWN')
*
     
*
      ELSE
*
         WRITE(6,2003)
 2003    FORMAT (2X,'PARTON INTERPOLATION: ISET OUT OF RANGE -- '
     1        //'STOP')
         STOP
*
      ENDIF
*
      
      DO IQ = 1,NQ
         DO IX = 1,NX
            READ(LIN,*) ,XB(IX), QS(IQ),
     1           (XPDF(IX,IQ,IPDF),IPDF=1,NPDF)

         ENDDO
      ENDDO
*     
      CLOSE(LIN)
*
*     CALCULATE SPLINE COEFFIFIENTS FOR THE INTERPOLATION IN X :
*
      DO IPDF = 1,NPDF
         DO IQ = 1,NQ
            CALL SPLINE(NX,XB,XPDF,AXB,BXB,CXB,IPDF,IQ)
            
         ENDDO
      ENDDO
*
      IINI = 1
*
 10   CONTINUE
*
*     INTERPOLATION :
*     X: CUBIC SPLINE INTERPOLATION, LOG(Q2): LINEAR INTERPOLATION
*
      IQ = ISEARCH(NQ,QS,Q2)
      IF (IQ .EQ. NQ) IQ = NQ-1
      IX = ISEARCH(NX,XB,X)
*
      DX = X - XB(IX)
*
      DO IPDF = 1,NPDF
         PDF1(IPDF) = XPDF(IX,IQ,IPDF)
     1        + DX*(AXB(IX,IQ,IPDF) + DX*(BXB(IX,IQ,IPDF) 
     2        + DX*CXB(IX,IQ,IPDF)))
         PDF2(IPDF) = XPDF(IX,IQ+1,IPDF)
     1        + DX*(AXB(IX,IQ+1,IPDF) + DX*(BXB(IX,IQ+1,IPDF) 
     2        + DX*CXB(IX,IQ+1,IPDF)))
        
      ENDDO
*
      TQ = (DLOG(Q2)-DLOG(QS(IQ))) / (DLOG(QS(IQ+1))-DLOG(QS(IQ)))
*
      DO IPDF = 1,NPDF
         PDF(IPDF)  = (1.0D0-TQ)*PDF1(IPDF) + TQ*PDF2(IPDF)
         
      ENDDO
*
      Uv    = PDF(1)
      Dv    = PDF(2)  
      Ubar  = PDF(3)
      Dbar  = PDF(4) 
      Sbar  = PDF(5)
      Gl    = PDF(6)
      DUv   = PDF(7)
      DDv   = PDF(8)
      DUbar = PDF(9)
      DDbar = PDF(10)
      DSbar = PDF(11)
      DGl   = PDF(12)
	G1p   = PDF(13)
	G1n   = PDF(14)
	G1d   = PDF(15)
	dG1p  = PDF(16)
	dG1n  = PDF(17)
	dG1d  = PDF(18)
        xg2   = PDF(19)
      RETURN
      END
*
* ---------------------------------------------------------------------
      SUBROUTINE SPLINE(N,X,Y,B,C,D,I,J)
* ---------------------------------------------------------------------
* CALCULATE THE COEFFICIENTS B,C,D IN A CUBIC SPLINE INTERPOLATION.
* INTERPOLATION SUBROUTINES ARE TAKEN FROM
* G.E. FORSYTHE, M.A. MALCOLM AND C.B. MOLER,
* COMPUTER METHODS FOR MATHEMATICAL COMPUTATIONS (PRENTICE-HALL, 1977).
*

*
      IMPLICIT REAL*8(A-H,O-Z)
*
      PARAMETER (NPDF=19, NQ=50, NX=100 )
*
      DIMENSION X(NX), Y(NX,NQ,NPDF),
     1          B(NX,NQ,NPDF), C(NX,NQ,NPDF), D(NX,NQ,NPDF)
*
      NM1=N-1
      IF(N.LT.2) RETURN
      IF(N.LT.3) GO TO 250
      D(1,J,I)=X(2)-X(1)
      C(2,J,I)=(Y(2,J,I)-Y(1,J,I))/D(1,J,I)
      DO 210 K=2,NM1
         D(K,J,I)=X(K+1)-X(K)
         B(K,J,I)=2.0D0*(D(K-1,J,I)+D(K,J,I))
         C(K+1,J,I)=(Y(K+1,J,I)-Y(K,J,I))/D(K,J,I)
         C(K,J,I)=C(K+1,J,I)-C(K,J,I)
  210 CONTINUE
      B(1,J,I)=-D(1,J,I)
      B(N,J,I)=-D(N-1,J,I)
      C(1,J,I)=0.0D0
      C(N,J,I)=0.0D0
      IF(N.EQ.3) GO TO 215
      C(1,J,I)=C(3,J,I)/(X(4)-X(2))-C(2,J,I)/(X(3)-X(1))
      C(N,J,I)=C(N-1,J,I)/(X(N)-X(N-2))-C(N-2,J,I)/(X(N-1)-X(N-3))
      C(1,J,I)=C(1,J,I)*D(1,J,I)**2.0D0/(X(4)-X(1))
      C(N,J,I)=-C(N,J,I)*D(N-1,J,I)**2.0D0/(X(N)-X(N-3))
 215  CONTINUE
      DO 220 K=2,N
         T=D(K-1,J,I)/B(K-1,J,I)
         B(K,J,I)=B(K,J,I)-T*D(K-1,J,I)
         C(K,J,I)=C(K,J,I)-T*C(K-1,J,I)
 220  CONTINUE
      C(N,J,I)=C(N,J,I)/B(N,J,I)
      DO 230 IB=1,NM1
         K=N-IB
         C(K,J,I)=(C(K,J,I)-D(K,J,I)*C(K+1,J,I))/B(K,J,I)
 230  CONTINUE
      B(N,J,I)=(Y(N,J,I)-Y(NM1,J,I))/D(NM1,J,I)
     1     +D(NM1,J,I)*(C(NM1,J,I)+2.0D0*C(N,J,I))
      DO 240 K=1,NM1
         B(K,J,I)=(Y(K+1,J,I)-Y(K,J,I))/D(K,J,I)
     1        -D(K,J,I)*(C(K+1,J,I)+2.0D0*C(K,J,I))
         D(K,J,I)=(C(K+1,J,I)-C(K,J,I))/D(K,J,I)
         C(K,J,I)=3.0D0*C(K,J,I)
 240  CONTINUE
      C(N,J,I)=3.0D0*C(N,J,I)
      D(N,J,I)=D(N-1,J,I)
      RETURN
 250  CONTINUE
      B(1,J,I)=(Y(2,J,I)-Y(1,J,I))/(X(2)-X(1))
      C(1,J,I)=0.0D0
      D(1,J,I)=0.0D0
      B(2,J,I)=B(1,J,I)
      C(2,J,I)=0.0D0
      D(2,J,I)=0.0D0
      RETURN
      END
*
* ---------------------------------------------------------------------
      INTEGER FUNCTION ISEARCH(N,X,Y)
* ---------------------------------------------------------------------
* THIS FUNCTION SEARCHES "I" WHICH SATISFIES THE RELATION
* X(I) <= Y < X(I+1) BY USING A BINARY SEARCH.
*

*
      IMPLICIT REAL*8(A-H,O-Z)
*
      PARAMETER (NPDF=19, NQ=50, NX=100 )
*
      DIMENSION X(NX)
*
      MIN=1
      MAX=N+1
*
   10 CONTINUE
      MID=(MIN+MAX)/2
      IF(Y.LT.X(MID)) THEN
        MAX=MID
      ELSE
        MIN=MID
      END IF
      IF((MAX-MIN).GT.1) GO TO 10
*
      ISEARCH=MIN
*
      RETURN
      END
*
* *********************************************************************
